@@ -85,8 +85,7 @@ notebook_get_name (GeditMultiNotebook *mnb,
 {
 	guint num;
 
-	num = gedit_multi_notebook_get_notebook_num (mnb,
-						     notebook);
+	num = gedit_multi_notebook_get_notebook_num (mnb, notebook);
 
 	return g_markup_printf_escaped ("Tab Group %i", num + 1);
 }
@@ -145,14 +144,14 @@ tab_get_name (GeditTab *tab)
 }
 
 static gboolean
-get_iters_from_notebook_and_tab (GeditDocumentsPanel *panel,
-				 GeditNotebook       *notebook,
-				 GtkTreeIter         *notebook_iter,
-				 GeditTab            *tab,
-				 GtkTreeIter         *tab_iter)
+get_iter_from_tab (GeditDocumentsPanel *panel,
+		   GeditNotebook       *notebook,
+		   GeditTab            *tab,
+		   GtkTreeIter         *tab_iter)
 {
-	gboolean success;
 	GtkTreeIter parent;
+	gboolean success;
+	gboolean search_notebook;
 
 	/* Note: we cannot use the functions of the MultiNotebook
 	 *       because in cases where the notebook or tab has been
@@ -162,62 +161,78 @@ get_iters_from_notebook_and_tab (GeditDocumentsPanel *panel,
 	gedit_debug (DEBUG_PANEL);
 
 	g_assert (notebook != NULL || tab != NULL);
-	g_assert (notebook_iter != NULL || tab_iter != NULL);
-
-	g_assert (notebook_iter == NULL || (notebook_iter != NULL && notebook != NULL));
+	g_assert (tab_iter != NULL);
 	g_assert ((tab == NULL && tab_iter == NULL) || (tab != NULL && tab_iter != NULL));
 
 	success = FALSE;
+	search_notebook = (gedit_multi_notebook_get_n_notebooks (panel->priv->mnb) > 1);
 
 	/* We never ever ask for a notebook or tab if it does not already exist in the tree */
 	g_assert (gtk_tree_model_get_iter_first (panel->priv->model, &parent));
 
 	do
 	{
-		GtkTreeIter iter;
-
-		if (notebook != NULL)
+		if (search_notebook)
 		{
 			GeditNotebook *current_notebook;
+			gboolean is_cur;
 
 			gtk_tree_model_get (panel->priv->model,
 					    &parent,
 					    NOTEBOOK_COLUMN, &current_notebook,
 					    -1);
-			g_assert (current_notebook != NULL);
 
-			if (current_notebook == notebook)
+			is_cur = (current_notebook == notebook);
+
+			g_object_unref (current_notebook);
+
+			if (is_cur)
 			{
 				success = TRUE;
 
-				if (notebook_iter != NULL)
-				{
-					*notebook_iter = parent;
-				}
-
 				if (tab == NULL)
 				{
 					break;
 				}
 			}
 		}
+		else if (tab == NULL)
+		{
+			success = TRUE;
 
-		if (notebook == NULL || success)
+			break;
+		}
+
+		if (success || !search_notebook)
 		{
 			GeditTab *current_tab;
+			GtkTreeIter iter;
 
-			/* We never ever ask for a tab if it does not already exist in the tree */
-			g_assert (gtk_tree_model_iter_children (panel->priv->model, &iter, &parent));
+			if (search_notebook)
+			{
+				g_assert (gtk_tree_model_iter_children (panel->priv->model,
+									&iter, &parent));
+			}
+			else
+			{
+				iter = parent;
+			}
 
 			do
 			{
+				gboolean is_cur;
+
 				gtk_tree_model_get (panel->priv->model,
 						    &iter,
 						    TAB_COLUMN, &current_tab,
 						    -1);
 				g_assert (current_tab != NULL);
 
-				if (current_tab == tab)
+				is_cur = (current_tab == tab);
+
+				g_object_unref (current_tab);
+
+				if (is_cur)
 				{
 					*tab_iter = iter;
 					success = TRUE;
@@ -237,13 +252,6 @@ out:
 	/* In the current code if we are not successful then something is wrong */
 	g_assert (success);
 
-	/* And therefore will always have valid iters for those that are asked for */
-	if (notebook_iter != NULL)
-	{
-		g_assert (gtk_tree_store_iter_is_valid (GTK_TREE_STORE (panel->priv->model),
-							notebook_iter));
-	}
-
 	if (tab_iter != NULL)
 	{
 		g_assert (gtk_tree_store_iter_is_valid (GTK_TREE_STORE (panel->priv->model),
@@ -253,16 +261,6 @@ out:
 	return success;
 }
 
-#define get_iter_from_notebook(panel, notebook, iter)		\
-	get_iters_from_notebook_and_tab ((panel),		\
-					 (notebook), (iter),	\
-					 NULL, NULL)
-
-#define get_iter_from_tab(panel, notebook, tab, iter)		\
-	get_iters_from_notebook_and_tab ((panel),		\
-					 (notebook), NULL,	\
-					 (tab), (iter))
-
 static void
 select_iter (GeditDocumentsPanel *panel,
 	     GtkTreeIter         *iter)
@@ -288,11 +286,13 @@ select_active_tab (GeditDocumentsPanel *panel)
 {
 	GeditNotebook *notebook;
 	GeditTab *tab;
+	gboolean have_tabs;
 
 	notebook = gedit_multi_notebook_get_active_notebook (panel->priv->mnb);
+	have_tabs = gtk_notebook_get_n_pages (GTK_NOTEBOOK (notebook)) > 0;
 	tab = gedit_multi_notebook_get_active_tab (panel->priv->mnb);
 
-	if (notebook != NULL && tab != NULL)
+	if (notebook != NULL && tab != NULL && have_tabs)
 	{
 		GtkTreeIter iter;
 
@@ -365,7 +365,9 @@ refresh_notebook (GeditDocumentsPanel *panel,
 
 		g_free (name);
 		if (pixbuf != NULL)
+		{
 			g_object_unref (pixbuf);
+		}
 
 		if (l->data == active_tab)
 		{
@@ -380,25 +382,38 @@ static void
 refresh_notebook_foreach (GeditNotebook       *notebook,
 			  GeditDocumentsPanel *panel)
 {
-	gchar *name;
-	GtkTreeIter iter;
+	gboolean add_notebook;
 
-	name = notebook_get_name (panel->priv->mnb, notebook);
+	/* If we have only one notebook we don't want to show the notebook
+	   header */
+	add_notebook = (gedit_multi_notebook_get_n_notebooks (panel->priv->mnb) > 1);
 
-	gtk_tree_store_append (GTK_TREE_STORE (panel->priv->model),
-			       &iter, NULL);
+	if (add_notebook)
+	{
+		GtkTreeIter iter;
+		gchar *name;
 
-	gtk_tree_store_set (GTK_TREE_STORE (panel->priv->model),
-			    &iter,
-			    PIXBUF_COLUMN, NULL,
-			    NAME_COLUMN, name,
-			    NOTEBOOK_COLUMN, notebook,
-			    TAB_COLUMN, NULL,
-			    -1);
+		name = notebook_get_name (panel->priv->mnb, notebook);
 
-	refresh_notebook (panel, notebook, &iter);
+		gtk_tree_store_append (GTK_TREE_STORE (panel->priv->model),
+				       &iter, NULL);
 
-	g_free (name);
+		gtk_tree_store_set (GTK_TREE_STORE (panel->priv->model),
+				    &iter,
+				    PIXBUF_COLUMN, NULL,
+				    NAME_COLUMN, name,
+				    NOTEBOOK_COLUMN, notebook,
+				    TAB_COLUMN, NULL,
+				    -1);
+
+		refresh_notebook (panel, notebook, &iter);
+
+		g_free (name);
+	}
+	else
+	{
+		refresh_notebook (panel, notebook, NULL);
+	}
 }
 
 static void
@@ -443,7 +458,9 @@ sync_name_and_icon (GeditTab            *tab,
 
 	gedit_debug (DEBUG_PANEL);
 
-	get_iter_from_tab (panel, NULL, tab, &iter);
+	get_iter_from_tab (panel,
+			   gedit_multi_notebook_get_active_notebook (panel->priv->mnb),
+			   tab, &iter);
 
 	name = tab_get_name (tab);
 	pixbuf = _gedit_tab_get_icon (tab);
@@ -456,7 +473,9 @@ sync_name_and_icon (GeditTab            *tab,
 
 	g_free (name);
 	if (pixbuf != NULL)
+	{
 		g_object_unref (pixbuf);
+	}
 }
 
 static void
@@ -475,11 +494,7 @@ multi_notebook_tab_removed (GeditMultiNotebook  *mnb,
 					      G_CALLBACK (sync_name_and_icon),
 					      panel);
 
-	GtkTreeIter iter;
-
-	get_iter_from_tab (panel, notebook, tab, &iter);
-	gtk_tree_store_remove (GTK_TREE_STORE (panel->priv->model),
-			       &iter);
+	refresh_list (panel);
 }
 
 static void
@@ -488,12 +503,6 @@ multi_notebook_tab_added (GeditMultiNotebook  *mnb,
 			  GeditTab            *tab,
 			  GeditDocumentsPanel *panel)
 {
-	GtkTreeIter iter;
-	GtkTreeIter parent;
-	GeditTab *active_tab;
-	GdkPixbuf *pixbuf;
-	gchar *name;
-
 	gedit_debug (DEBUG_PANEL);
 
 	g_signal_connect (tab,
@@ -505,39 +514,7 @@ multi_notebook_tab_added (GeditMultiNotebook  *mnb,
 			  G_CALLBACK (sync_name_and_icon),
 			  panel);
 
-	get_iter_from_notebook (panel, notebook, &parent);
-
-	panel->priv->adding_tab = TRUE;
-
-	gtk_tree_store_append (GTK_TREE_STORE (panel->priv->model),
-			       &iter,
-			       &parent);
-
-	active_tab = gedit_window_get_active_tab (panel->priv->window);
-
-	if (tab == active_tab)
-	{
-		select_iter (panel, &iter);
-	}
-
-	name = tab_get_name (tab);
-	pixbuf = _gedit_tab_get_icon (tab);
-
-	gtk_tree_store_set (GTK_TREE_STORE (panel->priv->model),
-			    &iter,
-		            PIXBUF_COLUMN, pixbuf,
-		            NAME_COLUMN, name,
-		            NOTEBOOK_COLUMN, notebook,
-		            TAB_COLUMN, tab,
-		            -1);
-
-	panel->priv->adding_tab = FALSE;
-
-	gtk_tree_view_expand_all (GTK_TREE_VIEW (panel->priv->treeview));
-
-	g_free (name);
-	if (pixbuf != NULL)
-		g_object_unref (pixbuf);
+	refresh_list (panel);
 }
 
 static void
@@ -545,81 +522,9 @@ multi_notebook_notebook_removed (GeditMultiNotebook  *mnb,
 				 GeditNotebook       *notebook,
 				 GeditDocumentsPanel *panel)
 {
-	GtkTreeIter iter;
-
-	gedit_debug (DEBUG_PANEL);
-
-	get_iter_from_notebook (panel, notebook, &iter);
-	gtk_tree_store_remove (GTK_TREE_STORE (panel->priv->model),
-			       &iter);
-
-	/* refresh the name of the notebooks */
-	if (gtk_tree_model_get_iter_first (panel->priv->model, &iter))
-	{
-		GeditNotebook *current_notebook;
-		gchar *notebook_name;
-
-		do
-		{
-			gtk_tree_model_get (panel->priv->model,
-					    &iter,
-					    NOTEBOOK_COLUMN, &current_notebook,
-					    -1);
-			g_assert (current_notebook != NULL);
-
-			notebook_name = notebook_get_name (mnb, current_notebook);
-			gtk_tree_store_set (GTK_TREE_STORE (panel->priv->model),
-					    &iter,
-					    NAME_COLUMN, notebook_name,
-					    -1);
-
-			g_free (notebook_name);
-		} while (gtk_tree_model_iter_next (panel->priv->model, &iter));
-	}
+	refresh_list (panel);
 }
 
-static void
-multi_notebook_notebook_added (GeditMultiNotebook  *mnb,
-			       GeditNotebook       *notebook,
-			       GeditDocumentsPanel *panel)
-{
-	GtkTreeIter iter;
-	GeditNotebook *active_notebook;
-	gchar *name;
-
-	gedit_debug (DEBUG_PANEL);
-
-	panel->priv->adding_tab = TRUE;
-
-	gtk_tree_store_append (GTK_TREE_STORE (panel->priv->model),
-			       &iter,
-			       NULL);
-
-	active_notebook = gedit_multi_notebook_get_active_notebook (panel->priv->mnb);
-
-	if (notebook == active_notebook)
-	{
-		select_iter (panel, &iter);
-	}
-
-	name = notebook_get_name (panel->priv->mnb, notebook);
-
-	gtk_tree_store_set (GTK_TREE_STORE (panel->priv->model),
-			    &iter,
-		            PIXBUF_COLUMN, NULL,
-		            NAME_COLUMN, name,
-		            NOTEBOOK_COLUMN, notebook,
-		            TAB_COLUMN, NULL,
-		            -1);
-
-	panel->priv->adding_tab = FALSE;
-
-	gtk_tree_view_expand_all (GTK_TREE_VIEW (panel->priv->treeview));
-
-	g_free (name);
-}
-
-/* TODO
 static void
 multi_notebook_tabs_reordered (GeditWindow         *window,
 			       GeditDocumentsPanel *panel)
@@ -630,8 +535,7 @@ multi_notebook_tabs_reordered (GeditWindow         *window,
 		return;
 
 	refresh_list (panel);
-} */
-
+}
 
 static void
 set_window (GeditDocumentsPanel *panel,
@@ -645,15 +549,10 @@ set_window (GeditDocumentsPanel *panel,
 	panel->priv->window = g_object_ref (window);
 	panel->priv->mnb = GEDIT_MULTI_NOTEBOOK (_gedit_window_get_multi_notebook (window));
 
-	g_signal_connect (panel->priv->mnb,
-			  "notebook-added",
-			  G_CALLBACK (multi_notebook_notebook_added),
-			  panel);
 	g_signal_connect (panel->priv->mnb,
 			  "notebook-removed",
 			  G_CALLBACK (multi_notebook_notebook_removed),
 			  panel);
-
 	g_signal_connect (panel->priv->mnb,
 			  "tab-added",
 			  G_CALLBACK (multi_notebook_tab_added),
@@ -662,11 +561,10 @@ set_window (GeditDocumentsPanel *panel,
 			  "tab-removed",
 			  G_CALLBACK (multi_notebook_tab_removed),
 			  panel);
-	/* TODO
 	g_signal_connect (panel->priv->mnb,
 			  "tabs-reordered",
-			  G_CALLBACK (multi_notebook_tab_reordered),
-			  panel); */
+			  G_CALLBACK (multi_notebook_tabs_reordered),
+			  panel);
 	g_signal_connect (panel->priv->mnb,
 			  "switch-tab",
 			  G_CALLBACK (multi_notebook_tab_switched),
@@ -701,6 +599,7 @@ treeview_cursor_changed (GtkTreeView         *view,
 		{
 			gedit_multi_notebook_set_active_tab (panel->priv->mnb,
 							     tab);
+			g_object_unref (tab);
 		}
 		else
 		{
@@ -859,7 +758,7 @@ menu_position (GtkMenu             *menu,
 
 	gdk_window_get_origin (gtk_widget_get_window (w), x, y);
 
-	gtk_widget_size_request (GTK_WIDGET (menu), &requisition);
+	gtk_widget_get_preferred_size (GTK_WIDGET (menu), &requisition, NULL);
 
 	gtk_widget_get_allocation (w, &allocation);
 
@@ -966,6 +865,8 @@ panel_button_press_event (GtkTreeView         *treeview,
 
 					/* A row exists at the mouse position */
 					ret = show_tab_popup_menu (panel, event);
+
+					g_object_unref (tab);
 				}
 			}
 
@@ -1053,6 +954,7 @@ treeview_query_tooltip (GtkWidget           *widget,
 	if (tab != NULL)
 	{
 		tip = _gedit_tab_get_tooltip (tab);
+		g_object_unref (tab);
 	}
 	else
 	{
@@ -1061,6 +963,7 @@ treeview_query_tooltip (GtkWidget           *widget,
 
 	gtk_tooltip_set_markup (tooltip, tip);
 
+	g_object_unref (notebook);
 	g_free (tip);
 	gtk_tree_path_free (path);
 
@@ -1078,60 +981,31 @@ treeview_row_inserted (GtkTreeModel        *tree_model,
 	gint *indeces;
 	gchar *path_string;
 	GeditNotebook *notebook;
-	ItemTypes item_type;
-	gint old_position;
-	gint new_position;
 
 	gedit_debug (DEBUG_PANEL);
 
 	if (panel->priv->adding_tab)
 		return;
 
-	tab = gedit_window_get_active_tab (panel->priv->window);
-	g_return_if_fail (tab != NULL);
-
 	panel->priv->is_reodering = TRUE;
 
-	indeces = gtk_tree_path_get_indices (path);
 	path_string = gtk_tree_path_to_string (path);
 
 	gedit_debug_message (DEBUG_PANEL, "New Path: %s", path_string);
 
-	notebook = gedit_multi_notebook_get_nth_notebook (panel->priv->mnb,
-							  indeces[0]);
+	g_message ("%s", path_string);
 
 	gtk_tree_model_get (panel->priv->model,
 			    iter,
-			    ITEM_TYPE_COLUMN, &item_type,
+			    NOTEBOOK_COLUMN, &notebook,
+			    TAB_COLUMN, &tab,
 			    -1);
 
-	switch (item_type)
-	{
-		case ITEM_TYPE_TAB:
-			new_position = indeces[1];
-			old_position = gtk_notebook_page_num (GTK_NOTEBOOK (notebook),
-						    	      GTK_WIDGET (tab));
-
-			if (new_position > old_position)
-				new_position = MAX (0, new_position - 1);
-
-			gedit_notebook_reorder_tab (GEDIT_NOTEBOOK (notebook),
-						    tab,
-						    new_position);
-
-		case ITEM_TYPE_NOTEBOOK:
-			/ * Do we want to allow reordering the notebooks? * /
-			break;
-
-		default:
-			g_assert_not_reached ();
-			break;
-	}
-
 	panel->priv->is_reodering = FALSE;
 
 	g_free (path_string);
-} */
+}
+*/
 
 static void
 pixbuf_data_func (GtkTreeViewColumn   *column,
@@ -1148,6 +1022,11 @@ pixbuf_data_func (GtkTreeViewColumn   *column,
 			    -1);
 
 	gtk_cell_renderer_set_visible (cell, tab != NULL);
+
+	if (tab != NULL)
+	{
+		g_object_unref (tab);
+	}
 }
 
 static void
@@ -1172,7 +1051,7 @@ gedit_documents_panel_init (GeditDocumentsPanel *panel)
 					GTK_POLICY_AUTOMATIC,
 					GTK_POLICY_AUTOMATIC);
 	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
-                                             GTK_SHADOW_IN);
+	                                     GTK_SHADOW_IN);
 	gtk_widget_show (sw);
 	gtk_box_pack_start (GTK_BOX (panel), sw, TRUE, TRUE, 0);
 
@@ -1188,7 +1067,7 @@ gedit_documents_panel_init (GeditDocumentsPanel *panel)
 	g_object_unref (G_OBJECT (panel->priv->model));
 	gtk_container_add (GTK_CONTAINER (sw), panel->priv->treeview);
 	gtk_tree_view_set_headers_visible (GTK_TREE_VIEW (panel->priv->treeview), FALSE);
-	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (panel->priv->treeview), FALSE /* TODO */);
+	gtk_tree_view_set_reorderable (GTK_TREE_VIEW (panel->priv->treeview), FALSE); /* TODO */
 	gtk_tree_view_set_show_expanders (GTK_TREE_VIEW (panel->priv->treeview), FALSE);
 	gtk_tree_view_set_level_indentation (GTK_TREE_VIEW (panel->priv->treeview), 18);
 
@@ -1237,11 +1116,11 @@ gedit_documents_panel_init (GeditDocumentsPanel *panel)
 			  G_CALLBACK (treeview_query_tooltip),
 			  panel);
 
-	/* TODO
+	/*
 	g_signal_connect (panel->priv->model,
 			  "row-inserted",
 			  G_CALLBACK (treeview_row_inserted),
-			  panel); */
+			  panel);*/
 }
 
 GtkWidget *
