@@ -125,16 +125,10 @@ create_option_menu (GeditFileChooserDialog *dialog)
 static void
 update_newline_visibility (GeditFileChooserDialog *dialog)
 {
-	if (gtk_file_chooser_get_action (GTK_FILE_CHOOSER (dialog)) == GTK_FILE_CHOOSER_ACTION_SAVE)
-	{
-		gtk_widget_show (dialog->priv->newline_label);
-		gtk_widget_show (dialog->priv->newline_combo);
-	}
-	else
-	{
-		gtk_widget_hide (dialog->priv->newline_label);
-		gtk_widget_hide (dialog->priv->newline_combo);
-	}
+	gboolean visible = gtk_file_chooser_get_action (GTK_FILE_CHOOSER (dialog)) == GTK_FILE_CHOOSER_ACTION_SAVE;
+
+	gtk_widget_set_visible (dialog->priv->newline_label, visible);
+	gtk_widget_set_visible (dialog->priv->newline_combo, visible);
 }
 
 static void
@@ -391,7 +385,6 @@ gedit_file_chooser_dialog_new_valist (const gchar          *title,
 
 	result = g_object_new (GEDIT_TYPE_FILE_CHOOSER_DIALOG,
 			       "title", title,
-			       "file-system-backend", NULL,
 			       "local-only", FALSE,
 			       "action", action,
 			       "select-multiple", action == GTK_FILE_CHOOSER_ACTION_OPEN,
@@ -472,11 +465,13 @@ gedit_file_chooser_dialog_new_valist (const gchar          *title,
 
 /**
  * gedit_file_chooser_dialog_new:
- * @title: Title of the dialog, or %NULL
- * @parent: Transient parent of the dialog, or %NULL
+ * @title: (allow-none): Title of the dialog, or %NULL
+ * @parent: (allow-none): Transient parent of the dialog, or %NULL
  * @action: Open or save mode for the dialog
- * @first_button_text: stock ID or text to go in the first button, or %NULL
- * @Varargs: response ID for the first button, then additional (button, id) pairs, ending with %NULL
+ * @first_button_text: (allow-none): stock ID or text to go in
+ * the first button, or %NULL
+ * @Varargs: (allow-none): response ID for the first button, then
+ * additional (button, id) pairs, ending with %NULL
  *
  * Creates a new #GeditFileChooserDialog.  This function is analogous to
  * gtk_dialog_new_with_buttons().
