@@ -37,17 +37,14 @@
 #include "gedit-notebook.h"
 #include "gedit-tab.h"
 #include "gedit-utils.h"
-#include "gedit-io-error-message-area.h"
+#include "gedit-io-error-info-bar.h"
 #include "gedit-print-job.h"
 #include "gedit-print-preview.h"
-#include "gedit-progress-message-area.h"
+#include "gedit-progress-info-bar.h"
 #include "gedit-debug.h"
 #include "gedit-enum-types.h"
 #include "gedit-settings.h"
-
-#if !GTK_CHECK_VERSION (2, 17, 1)
-#include "gedit-message-area.h"
-#endif
+#include "gedit-view-frame.h"
 
 #define GEDIT_TAB_GET_PRIVATE(object)(G_TYPE_INSTANCE_GET_PRIVATE ((object), GEDIT_TYPE_TAB, GeditTabPrivate))
 
@@ -57,11 +54,10 @@ struct _GeditTabPrivate
 {
 	GSettings	       *editor;
 	GeditTabState	        state;
-	
-	GtkWidget	       *view;
-	GtkWidget	       *view_scrolled_window;
 
-	GtkWidget	       *message_area;
+	GeditViewFrame         *frame;
+
+	GtkWidget	       *info_bar;
 	GtkWidget	       *print_preview;
 
 	GeditPrintJob          *print_job;
@@ -99,6 +95,15 @@ enum
 	PROP_AUTO_SAVE_INTERVAL
 };
 
+/* Signals */
+enum
+{
+	DROP_URIS,
+	LAST_SIGNAL
+};
+
+static guint signals[LAST_SIGNAL] = { 0 };
+
 static gboolean gedit_tab_auto_save (GeditTab *tab);
 
 static void
@@ -267,9 +272,9 @@ gedit_tab_grab_focus (GtkWidget *widget)
 	
 	GTK_WIDGET_CLASS (gedit_tab_parent_class)->grab_focus (widget);
 	
-	if (tab->priv->message_area != NULL)
+	if (tab->priv->info_bar != NULL)
 	{
-		gtk_widget_grab_focus (tab->priv->message_area);
+		gtk_widget_grab_focus (tab->priv->info_bar);
 	}
 	else
 	{
@@ -285,7 +290,6 @@ gedit_tab_class_init (GeditTabClass *klass)
 
 	object_class->dispose = gedit_tab_dispose;
 	object_class->finalize = gedit_tab_finalize;
-	object_class->dispose = gedit_tab_dispose;
 	object_class->get_property = gedit_tab_get_property;
 	object_class->set_property = gedit_tab_set_property;
 	
@@ -330,6 +334,17 @@ gedit_tab_class_init (GeditTabClass *klass)
 							   G_PARAM_READWRITE |
 							   G_PARAM_STATIC_STRINGS));
 
+	signals[DROP_URIS] =
+		g_signal_new ("drop-uris",
+			      G_TYPE_FROM_CLASS (object_class),
+			      G_SIGNAL_RUN_LAST | G_SIGNAL_ACTION,
+			      G_STRUCT_OFFSET (GeditTabClass, drop_uris),
+			      NULL, NULL,
+			      g_cclosure_marshal_VOID__BOXED,
+			      G_TYPE_NONE,
+			      1,
+			      G_TYPE_STRV);
+
 	g_type_class_add_private (object_class, sizeof (GeditTabPrivate));
 }
 
@@ -404,25 +419,28 @@ static void
 set_view_properties_according_to_state (GeditTab      *tab,
 					GeditTabState  state)
 {
+	GeditView *view;
 	gboolean val;
 	gboolean hl_current_line;
 	
 	hl_current_line = g_settings_get_boolean (tab->priv->editor,
 						  GEDIT_SETTINGS_HIGHLIGHT_CURRENT_LINE);
 
+	view = gedit_view_frame_get_view (tab->priv->frame);
+
 	val = ((state == GEDIT_TAB_STATE_NORMAL) &&
 	       (tab->priv->print_preview == NULL) &&
 	       !tab->priv->not_editable);
-	gtk_text_view_set_editable (GTK_TEXT_VIEW (tab->priv->view), val);
+	gtk_text_view_set_editable (GTK_TEXT_VIEW (view), val);
 
 	val = ((state != GEDIT_TAB_STATE_LOADING) &&
 	       (state != GEDIT_TAB_STATE_CLOSING));
-	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (tab->priv->view), val);
+	gtk_text_view_set_cursor_visible (GTK_TEXT_VIEW (view), val);
 
 	val = ((state != GEDIT_TAB_STATE_LOADING) &&
 	       (state != GEDIT_TAB_STATE_CLOSING) &&
 	       (hl_current_line));
-	gtk_source_view_set_highlight_current_line (GTK_SOURCE_VIEW (tab->priv->view), val);
+	gtk_source_view_set_highlight_current_line (GTK_SOURCE_VIEW (view), val);
 }
 
 static void
@@ -442,18 +460,18 @@ gedit_tab_set_state (GeditTab      *tab,
 	if ((state == GEDIT_TAB_STATE_LOADING_ERROR) || /* FIXME: add other states if needed */
 	    (state == GEDIT_TAB_STATE_SHOWING_PRINT_PREVIEW))
 	{
-		gtk_widget_hide (tab->priv->view_scrolled_window);
+		gtk_widget_hide (GTK_WIDGET (tab->priv->frame));
 	}
 	else
 	{
 		if (tab->priv->print_preview == NULL)
-			gtk_widget_show (tab->priv->view_scrolled_window);
+			gtk_widget_show (GTK_WIDGET (tab->priv->frame));
 	}
 
-	set_cursor_according_to_state (GTK_TEXT_VIEW (tab->priv->view),
+	set_cursor_according_to_state (GTK_TEXT_VIEW (gedit_view_frame_get_view (tab->priv->frame)),
 				       state);
 
-	g_object_notify (G_OBJECT (tab), "state");		
+	g_object_notify (G_OBJECT (tab), "state");
 }
 
 static void 
@@ -486,28 +504,28 @@ document_modified_changed (GtkTextBuffer *document,
 }
 
 static void
-set_message_area (GeditTab  *tab,
-		  GtkWidget *message_area)
+set_info_bar (GeditTab  *tab,
+	      GtkWidget *info_bar)
 {
-	if (tab->priv->message_area == message_area)
+	if (tab->priv->info_bar == info_bar)
 		return;
 
-	if (tab->priv->message_area != NULL)
-		gtk_widget_destroy (tab->priv->message_area);
+	if (tab->priv->info_bar != NULL)
+		gtk_widget_destroy (tab->priv->info_bar);
 
-	tab->priv->message_area = message_area;
+	tab->priv->info_bar = info_bar;
 
-	if (message_area == NULL)
+	if (info_bar == NULL)
 		return;
 
 	gtk_box_pack_start (GTK_BOX (tab),
-			    tab->priv->message_area,
+			    tab->priv->info_bar,
 			    FALSE,
 			    FALSE,
 			    0);		
 
-	g_object_add_weak_pointer (G_OBJECT (tab->priv->message_area), 
-				   (gpointer *)&tab->priv->message_area);
+	g_object_add_weak_pointer (G_OBJECT (tab->priv->info_bar), 
+				   (gpointer *)&tab->priv->info_bar);
 }
 
 static void
@@ -521,9 +539,9 @@ remove_tab (GeditTab *tab)
 }
 
 static void 
-io_loading_error_message_area_response (GtkWidget *message_area,
-					gint       response_id,
-					GeditTab  *tab)
+io_loading_error_info_bar_response (GtkWidget *info_bar,
+				    gint       response_id,
+				    GeditTab  *tab)
 {
 	GeditDocument *doc;
 	GeditView *view;
@@ -543,15 +561,15 @@ io_loading_error_message_area_response (GtkWidget *message_area,
 		case GTK_RESPONSE_OK:
 			g_return_if_fail (location != NULL);
 
-			encoding = gedit_conversion_error_message_area_get_encoding (
-					GTK_WIDGET (message_area));
+			encoding = gedit_conversion_error_info_bar_get_encoding (
+					GTK_WIDGET (info_bar));
 
 			if (encoding != NULL)
 			{
 				tab->priv->tmp_encoding = encoding;
 			}
 
-			set_message_area (tab, NULL);
+			set_info_bar (tab, NULL);
 			gedit_tab_set_state (tab, GEDIT_TAB_STATE_LOADING);
 
 			g_return_if_fail (tab->priv->auto_save_timeout <= 0);
@@ -565,12 +583,12 @@ io_loading_error_message_area_response (GtkWidget *message_area,
 			break;
 		case GTK_RESPONSE_YES:
 			/* This means that we want to edit the document anyway */
-			set_message_area (tab, NULL);
+			set_info_bar (tab, NULL);
 			_gedit_document_set_readonly (doc, FALSE);
 			break;
 		case GTK_RESPONSE_NO:
 			/* We don't want to edit the document just show it */
-			set_message_area (tab, NULL);
+			set_info_bar (tab, NULL);
 			break;
 		default:
 			if (location != NULL)
@@ -589,9 +607,9 @@ io_loading_error_message_area_response (GtkWidget *message_area,
 }
 
 static void 
-file_already_open_warning_message_area_response (GtkWidget   *message_area,
-						 gint         response_id,
-						 GeditTab    *tab)
+file_already_open_warning_info_bar_response (GtkWidget   *info_bar,
+					     gint         response_id,
+					     GeditTab    *tab)
 {
 	GeditView *view;
 	
@@ -605,17 +623,17 @@ file_already_open_warning_message_area_response (GtkWidget   *message_area,
 					    TRUE);
 	}
 
-	gtk_widget_destroy (message_area);
+	gtk_widget_destroy (info_bar);
 
 	gtk_widget_grab_focus (GTK_WIDGET (view));
 }
 
 static void
-load_cancelled (GtkWidget        *area,
+load_cancelled (GtkWidget        *bar,
                 gint              response_id,
                 GeditTab         *tab)
 {
-	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
+	g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->priv->info_bar));
 	
 	g_object_ref (tab);
 	gedit_document_load_cancel (gedit_tab_get_document (tab));
@@ -623,16 +641,16 @@ load_cancelled (GtkWidget        *area,
 }
 
 static void 
-unrecoverable_reverting_error_message_area_response (GtkWidget        *message_area,
-						     gint              response_id,
-						     GeditTab         *tab)
+unrecoverable_reverting_error_info_bar_response (GtkWidget        *info_bar,
+						 gint              response_id,
+						 GeditTab         *tab)
 {
 	GeditView *view;
 	
 	gedit_tab_set_state (tab,
 			     GEDIT_TAB_STATE_NORMAL);
 
-	set_message_area (tab, NULL);
+	set_info_bar (tab, NULL);
 
 	view = gedit_tab_get_view (tab);
 
@@ -644,9 +662,9 @@ unrecoverable_reverting_error_message_area_response (GtkWidget        *message_a
 #define MAX_MSG_LENGTH 100
 
 static void
-show_loading_message_area (GeditTab *tab)
+show_loading_info_bar (GeditTab *tab)
 {
-	GtkWidget *area;
+	GtkWidget *bar;
 	GeditDocument *doc = NULL;
 	gchar *name;
 	gchar *dirname = NULL;
@@ -655,7 +673,7 @@ show_loading_message_area (GeditTab *tab)
 	gchar *dirname_markup;
 	gint len;
 
-	if (tab->priv->message_area != NULL)
+	if (tab->priv->info_bar != NULL)
 		return;
 
 	gedit_debug (DEBUG_TAB);
@@ -722,9 +740,9 @@ show_loading_message_area (GeditTab *tab)
 					       name_markup);
 		}
 		
-		area = gedit_progress_message_area_new (GTK_STOCK_REVERT_TO_SAVED,
-							msg,
-							TRUE);
+		bar = gedit_progress_info_bar_new (GTK_STOCK_REVERT_TO_SAVED,
+						   msg,
+						   TRUE);
 	}
 	else
 	{
@@ -745,19 +763,19 @@ show_loading_message_area (GeditTab *tab)
 					       name_markup);
 		}
 
-		area = gedit_progress_message_area_new (GTK_STOCK_OPEN,
-							msg,
-							TRUE);
+		bar = gedit_progress_info_bar_new (GTK_STOCK_OPEN,
+						   msg,
+						   TRUE);
 	}
 
-	g_signal_connect (area,
+	g_signal_connect (bar,
 			  "response",
 			  G_CALLBACK (load_cancelled),
 			  tab);
 						 
-	gtk_widget_show (area);
+	gtk_widget_show (bar);
 
-	set_message_area (tab, area);
+	set_info_bar (tab, bar);
 
 	g_free (msg);
 	g_free (name);
@@ -766,9 +784,9 @@ show_loading_message_area (GeditTab *tab)
 }
 
 static void
-show_saving_message_area (GeditTab *tab)
+show_saving_info_bar (GeditTab *tab)
 {
-	GtkWidget *area;
+	GtkWidget *bar;
 	GeditDocument *doc = NULL;
 	gchar *short_name;
 	gchar *from;
@@ -780,7 +798,7 @@ show_saving_message_area (GeditTab *tab)
 
 	g_return_if_fail (tab->priv->tmp_save_location != NULL);
 	
-	if (tab->priv->message_area != NULL)
+	if (tab->priv->info_bar != NULL)
 		return;
 	
 	gedit_debug (DEBUG_TAB);
@@ -806,9 +824,7 @@ show_saving_message_area (GeditTab *tab)
 		gchar *str;
 
 		from = short_name;
-
-		to = gedit_utils_uri_for_display (tab->priv->tmp_save_location);
-
+		to = g_file_get_parse_name (tab->priv->tmp_save_location);
 		str = gedit_utils_str_middle_truncate (to, 
 						       MAX (20, MAX_MSG_LENGTH - len));
 		g_free (to);
@@ -834,13 +850,13 @@ show_saving_message_area (GeditTab *tab)
 		msg = g_strdup_printf (_("Saving %s"), from_markup);
 	}
 
-	area = gedit_progress_message_area_new (GTK_STOCK_SAVE,
-						msg,
-						FALSE);
+	bar = gedit_progress_info_bar_new (GTK_STOCK_SAVE,
+					   msg,
+					   FALSE);
 
-	gtk_widget_show (area);
+	gtk_widget_show (bar);
 
-	set_message_area (tab, area);
+	set_info_bar (tab, bar);
 
 	g_free (msg);
 	g_free (to);
@@ -849,25 +865,25 @@ show_saving_message_area (GeditTab *tab)
 }
 
 static void
-message_area_set_progress (GeditTab *tab,
+info_bar_set_progress (GeditTab *tab,
 			   goffset   size,
 			   goffset   total_size)
 {
-	if (tab->priv->message_area == NULL)
+	if (tab->priv->info_bar == NULL)
 		return;
 
 	gedit_debug_message (DEBUG_TAB, "%" G_GUINT64_FORMAT "/%" G_GUINT64_FORMAT, size, total_size);
 
-	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
+	g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->priv->info_bar));
 	
 	if (total_size == 0)
 	{
 		if (size != 0)
-			gedit_progress_message_area_pulse (
-					GEDIT_PROGRESS_MESSAGE_AREA (tab->priv->message_area));	
+			gedit_progress_info_bar_pulse (
+					GEDIT_PROGRESS_INFO_BAR (tab->priv->info_bar));	
 		else
-			gedit_progress_message_area_set_fraction (
-				GEDIT_PROGRESS_MESSAGE_AREA (tab->priv->message_area),
+			gedit_progress_info_bar_set_fraction (
+				GEDIT_PROGRESS_INFO_BAR (tab->priv->info_bar),
 				0);
 	}
 	else
@@ -876,8 +892,8 @@ message_area_set_progress (GeditTab *tab,
 
 		frac = (gdouble)size / (gdouble)total_size;
 
-		gedit_progress_message_area_set_fraction (
-				GEDIT_PROGRESS_MESSAGE_AREA (tab->priv->message_area),
+		gedit_progress_info_bar_set_fraction (
+				GEDIT_PROGRESS_INFO_BAR (tab->priv->info_bar),
 				frac);		
 	}
 }
@@ -909,10 +925,10 @@ document_loading (GeditDocument *document,
 
 	if ((total_time - et) > 3.0)
 	{
-		show_loading_message_area (tab);
+		show_loading_info_bar (tab);
 	}
 
-	message_area_set_progress (tab, size, total_size);
+	info_bar_set_progress (tab, size, total_size);
 }
 
 static gboolean
@@ -943,7 +959,7 @@ document_loaded (GeditDocument *document,
 	}
 	tab->priv->times_called = 0;
 
-	set_message_area (tab, NULL);
+	set_info_bar (tab, NULL);
 
 	location = gedit_document_get_location (document);
 
@@ -978,37 +994,32 @@ document_loaded (GeditDocument *document,
 
 			if (tab->priv->state == GEDIT_TAB_STATE_LOADING_ERROR)
 			{
-				emsg = gedit_io_loading_error_message_area_new (location,
+				emsg = gedit_io_loading_error_info_bar_new (location,
 										tab->priv->tmp_encoding,
 										error);
 				g_signal_connect (emsg,
 						  "response",
-						  G_CALLBACK (io_loading_error_message_area_response),
+						  G_CALLBACK (io_loading_error_info_bar_response),
 						  tab);
 			}
 			else
 			{
 				g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_REVERTING_ERROR);
 				
-				emsg = gedit_unrecoverable_reverting_error_message_area_new (location,
+				emsg = gedit_unrecoverable_reverting_error_info_bar_new (location,
 											     error);
 
 				g_signal_connect (emsg,
 						  "response",
-						  G_CALLBACK (unrecoverable_reverting_error_message_area_response),
+						  G_CALLBACK (unrecoverable_reverting_error_info_bar_response),
 						  tab);
 			}
 
-			set_message_area (tab, emsg);
+			set_info_bar (tab, emsg);
 		}
 
-#if !GTK_CHECK_VERSION (2, 17, 1)
-		gedit_message_area_set_default_response (GEDIT_MESSAGE_AREA (emsg),
-							 GTK_RESPONSE_CANCEL);
-#else
 		gtk_info_bar_set_default_response (GTK_INFO_BAR (emsg),
 						   GTK_RESPONSE_CANCEL);
-#endif
 
 		gtk_widget_show (emsg);
 
@@ -1043,30 +1054,25 @@ document_loaded (GeditDocument *document,
 
 			_gedit_document_set_readonly (document, TRUE);
 
-			emsg = gedit_io_loading_error_message_area_new (location,
+			emsg = gedit_io_loading_error_info_bar_new (location,
 									tab->priv->tmp_encoding,
 									error);
 
-			set_message_area (tab, emsg);
+			set_info_bar (tab, emsg);
 
 			g_signal_connect (emsg,
 					  "response",
-					  G_CALLBACK (io_loading_error_message_area_response),
+					  G_CALLBACK (io_loading_error_info_bar_response),
 					  tab);
 
-#if !GTK_CHECK_VERSION (2, 17, 1)
-			gedit_message_area_set_default_response (GEDIT_MESSAGE_AREA (emsg),
-								 GTK_RESPONSE_CANCEL);
-#else
 			gtk_info_bar_set_default_response (GTK_INFO_BAR (emsg),
 							   GTK_RESPONSE_CANCEL);
-#endif
 
 			gtk_widget_show (emsg);
 		}
 
 		/* Scroll to the cursor when the document is loaded */
-		gedit_view_scroll_to_cursor (GEDIT_VIEW (tab->priv->view));
+		gedit_view_scroll_to_cursor (gedit_view_frame_get_view (tab->priv->frame));
 
 		all_documents = gedit_app_get_documents (gedit_app_get_default ());
 
@@ -1087,23 +1093,18 @@ document_loaded (GeditDocument *document,
 
 			    		tab->priv->not_editable = TRUE;
 
-			    		w = gedit_file_already_open_warning_message_area_new (location);
+			    		w = gedit_file_already_open_warning_info_bar_new (location);
 
-					set_message_area (tab, w);
+					set_info_bar (tab, w);
 
-#if !GTK_CHECK_VERSION (2, 17, 1)
-					gedit_message_area_set_default_response (GEDIT_MESSAGE_AREA (w),
-										 GTK_RESPONSE_CANCEL);
-#else
 					gtk_info_bar_set_default_response (GTK_INFO_BAR (w),
 									   GTK_RESPONSE_CANCEL);
-#endif
 
 					gtk_widget_show (w);
 
 					g_signal_connect (w,
 							  "response",
-							  G_CALLBACK (file_already_open_warning_message_area_response),
+							  G_CALLBACK (file_already_open_warning_info_bar_response),
 							  tab);
 
 			    		g_object_unref (loc);
@@ -1168,10 +1169,10 @@ document_saving (GeditDocument *document,
 
 	if ((total_time - et) > 3.0)
 	{
-		show_saving_message_area (tab);
+		show_saving_info_bar (tab);
 	}
 
-	message_area_set_progress (tab, size, total_size);
+	info_bar_set_progress (tab, size, total_size);
 
 	tab->priv->times_called++;
 }
@@ -1191,7 +1192,7 @@ end_saving (GeditTab *tab)
 }
 
 static void 
-unrecoverable_saving_error_message_area_response (GtkWidget        *message_area,
+unrecoverable_saving_error_info_bar_response (GtkWidget        *info_bar,
 						  gint              response_id,
 						  GeditTab         *tab)
 {
@@ -1204,7 +1205,7 @@ unrecoverable_saving_error_message_area_response (GtkWidget        *message_area
 
 	end_saving (tab);
 	
-	set_message_area (tab, NULL);
+	set_info_bar (tab, NULL);
 
 	view = gedit_tab_get_view (tab);
 
@@ -1212,7 +1213,7 @@ unrecoverable_saving_error_message_area_response (GtkWidget        *message_area
 }
 
 static void 
-no_backup_error_message_area_response (GtkWidget *message_area,
+no_backup_error_info_bar_response (GtkWidget *info_bar,
 				       gint       response_id,
 				       GeditTab  *tab)
 {
@@ -1223,7 +1224,7 @@ no_backup_error_message_area_response (GtkWidget *message_area,
 		doc = gedit_tab_get_document (tab);
 		g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 
-		set_message_area (tab, NULL);
+		set_info_bar (tab, NULL);
 
 		g_return_if_fail (tab->priv->tmp_save_location != NULL);
 		g_return_if_fail (tab->priv->tmp_encoding != NULL);
@@ -1240,14 +1241,14 @@ no_backup_error_message_area_response (GtkWidget *message_area,
 	}
 	else
 	{
-		unrecoverable_saving_error_message_area_response (message_area,
+		unrecoverable_saving_error_info_bar_response (info_bar,
 								  response_id,
 								  tab);
 	}
 }
 
 static void
-externally_modified_error_message_area_response (GtkWidget *message_area,
+externally_modified_error_info_bar_response (GtkWidget *info_bar,
 						 gint       response_id,
 						 GeditTab  *tab)
 {
@@ -1258,7 +1259,7 @@ externally_modified_error_message_area_response (GtkWidget *message_area,
 		doc = gedit_tab_get_document (tab);
 		g_return_if_fail (GEDIT_IS_DOCUMENT (doc));
 		
-		set_message_area (tab, NULL);
+		set_info_bar (tab, NULL);
 
 		g_return_if_fail (tab->priv->tmp_save_location != NULL);
 		g_return_if_fail (tab->priv->tmp_encoding != NULL);
@@ -1274,14 +1275,14 @@ externally_modified_error_message_area_response (GtkWidget *message_area,
 	}
 	else
 	{		
-		unrecoverable_saving_error_message_area_response (message_area,
+		unrecoverable_saving_error_info_bar_response (info_bar,
 								  response_id,
 								  tab);
 	}
 }
 
 static void 
-recoverable_saving_error_message_area_response (GtkWidget *message_area,
+recoverable_saving_error_info_bar_response (GtkWidget *info_bar,
 						gint       response_id,
 						GeditTab  *tab)
 {
@@ -1295,12 +1296,12 @@ recoverable_saving_error_message_area_response (GtkWidget *message_area,
 		const GeditEncoding *encoding;
 		gchar *tmp_uri;
 		
-		encoding = gedit_conversion_error_message_area_get_encoding (
-									GTK_WIDGET (message_area));
+		encoding = gedit_conversion_error_info_bar_get_encoding (
+									GTK_WIDGET (info_bar));
 
 		g_return_if_fail (encoding != NULL);
 
-		set_message_area (tab, NULL);
+		set_info_bar (tab, NULL);
 
 		g_return_if_fail (tab->priv->tmp_save_location != NULL);
 				
@@ -1323,7 +1324,7 @@ recoverable_saving_error_message_area_response (GtkWidget *message_area,
 	}
 	else
 	{		
-		unrecoverable_saving_error_message_area_response (message_area,
+		unrecoverable_saving_error_info_bar_response (info_bar,
 								  response_id,
 								  tab);
 	}
@@ -1346,7 +1347,7 @@ document_saved (GeditDocument *document,
 	tab->priv->timer = NULL;
 	tab->priv->times_called = 0;
 	
-	set_message_area (tab, NULL);
+	set_info_bar (tab, NULL);
 	
 	if (error != NULL)
 	{
@@ -1356,16 +1357,16 @@ document_saved (GeditDocument *document,
 		    error->code == GEDIT_DOCUMENT_ERROR_EXTERNALLY_MODIFIED)
 		{
 			/* This error is recoverable */
-			emsg = gedit_externally_modified_saving_error_message_area_new (
+			emsg = gedit_externally_modified_saving_error_info_bar_new (
 							tab->priv->tmp_save_location,
 							error);
 			g_return_if_fail (emsg != NULL);
 
-			set_message_area (tab, emsg);
+			set_info_bar (tab, emsg);
 
 			g_signal_connect (emsg,
 					  "response",
-					  G_CALLBACK (externally_modified_error_message_area_response),
+					  G_CALLBACK (externally_modified_error_info_bar_response),
 					  tab);
 		}
 		else if ((error->domain == GEDIT_DOCUMENT_ERROR &&
@@ -1374,16 +1375,16 @@ document_saved (GeditDocument *document,
 			  error->code == G_IO_ERROR_CANT_CREATE_BACKUP))
 		{
 			/* This error is recoverable */
-			emsg = gedit_no_backup_saving_error_message_area_new (
+			emsg = gedit_no_backup_saving_error_info_bar_new (
 							tab->priv->tmp_save_location,
 							error);
 			g_return_if_fail (emsg != NULL);
 
-			set_message_area (tab, emsg);
+			set_info_bar (tab, emsg);
 
 			g_signal_connect (emsg,
 					  "response",
-					  G_CALLBACK (no_backup_error_message_area_response),
+					  G_CALLBACK (no_backup_error_info_bar_response),
 					  tab);
 		}
 		else if (error->domain == GEDIT_DOCUMENT_ERROR ||
@@ -1395,15 +1396,15 @@ document_saved (GeditDocument *document,
 			_gedit_recent_remove  (GEDIT_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab))),
 					       tab->priv->tmp_save_location);
 
-			emsg = gedit_unrecoverable_saving_error_message_area_new (tab->priv->tmp_save_location,
+			emsg = gedit_unrecoverable_saving_error_info_bar_new (tab->priv->tmp_save_location,
 								  error);
 			g_return_if_fail (emsg != NULL);
 	
-			set_message_area (tab, emsg);
+			set_info_bar (tab, emsg);
 
 			g_signal_connect (emsg,
 					  "response",
-					  G_CALLBACK (unrecoverable_saving_error_message_area_response),
+					  G_CALLBACK (unrecoverable_saving_error_info_bar_response),
 					  tab);
 		}
 		else
@@ -1412,26 +1413,21 @@ document_saved (GeditDocument *document,
 			g_return_if_fail (error->domain == G_CONVERT_ERROR ||
 			                  error->domain == G_IO_ERROR);
 
-			emsg = gedit_conversion_error_while_saving_message_area_new (
+			emsg = gedit_conversion_error_while_saving_info_bar_new (
 									tab->priv->tmp_save_location,
 									tab->priv->tmp_encoding,
 									error);
 
-			set_message_area (tab, emsg);
+			set_info_bar (tab, emsg);
 
 			g_signal_connect (emsg,
 					  "response",
-					  G_CALLBACK (recoverable_saving_error_message_area_response),
+					  G_CALLBACK (recoverable_saving_error_info_bar_response),
 					  tab);
 		}
 
-#if !GTK_CHECK_VERSION (2, 17, 1)
-		gedit_message_area_set_default_response (GEDIT_MESSAGE_AREA (emsg),
-							 GTK_RESPONSE_CANCEL);
-#else
 		gtk_info_bar_set_default_response (GTK_INFO_BAR (emsg),
 						   GTK_RESPONSE_CANCEL);
-#endif
 
 		gtk_widget_show (emsg);
 	}
@@ -1456,13 +1452,13 @@ document_saved (GeditDocument *document,
 }
 
 static void 
-externally_modified_notification_message_area_response (GtkWidget *message_area,
+externally_modified_notification_info_bar_response (GtkWidget *info_bar,
 							gint       response_id,
 							GeditTab  *tab)
 {
 	GeditView *view;
 
-	set_message_area (tab, NULL);
+	set_info_bar (tab, NULL);
 	view = gedit_tab_get_view (tab);
 
 	if (response_id == GTK_RESPONSE_OK)
@@ -1483,7 +1479,7 @@ externally_modified_notification_message_area_response (GtkWidget *message_area,
 static void
 display_externally_modified_notification (GeditTab *tab)
 {
-	GtkWidget *message_area;
+	GtkWidget *info_bar;
 	GeditDocument *doc;
 	GFile *location;
 	gboolean document_modified;
@@ -1496,16 +1492,16 @@ display_externally_modified_notification (GeditTab *tab)
 	g_return_if_fail (location != NULL);
 
 	document_modified = gtk_text_buffer_get_modified (GTK_TEXT_BUFFER(doc));
-	message_area = gedit_externally_modified_message_area_new (location, document_modified);
+	info_bar = gedit_externally_modified_info_bar_new (location, document_modified);
 	g_object_unref (location);
 
-	tab->priv->message_area = NULL;
-	set_message_area (tab, message_area);
-	gtk_widget_show (message_area);
+	tab->priv->info_bar = NULL;
+	set_info_bar (tab, info_bar);
+	gtk_widget_show (info_bar);
 
-	g_signal_connect (message_area,
+	g_signal_connect (info_bar,
 			  "response",
-			  G_CALLBACK (externally_modified_notification_message_area_response),
+			  G_CALLBACK (externally_modified_notification_info_bar_response),
 			  tab);
 }
 
@@ -1550,25 +1546,22 @@ view_focused_in (GtkWidget     *widget,
 	return FALSE;
 }
 
-static GMountOperation *
-tab_mount_operation_factory (GeditDocument *doc,
-			     gpointer       userdata)
+static void
+on_drop_uris (GeditView *view,
+	      gchar    **uri_list,
+	      GeditTab  *tab)
 {
-	GeditTab *tab = GEDIT_TAB (userdata);
-	GtkWidget *window;
-
-	window = gtk_widget_get_toplevel (GTK_WIDGET (tab));
-	return gtk_mount_operation_new (GTK_WINDOW (window));
+	g_signal_emit (G_OBJECT (tab), signals[DROP_URIS], 0, uri_list);
 }
 
 static void
 gedit_tab_init (GeditTab *tab)
 {
-	GtkWidget *sw;
-	GeditDocument *doc;
 	GeditLockdownMask lockdown;
 	gboolean auto_save;
 	gint auto_save_interval;
+	GeditDocument *doc;
+	GeditView *view;
 
 	tab->priv = GEDIT_TAB_GET_PRIVATE (tab);
 
@@ -1581,14 +1574,6 @@ gedit_tab_init (GeditTab *tab)
 	tab->priv->save_flags = 0;
 
 	tab->priv->ask_if_externally_modified = TRUE;
-	
-	/* Create the scrolled window */
-	sw = gtk_scrolled_window_new (NULL, NULL);
-	tab->priv->view_scrolled_window = sw;
-
-	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (sw),
-					GTK_POLICY_AUTOMATIC,
-					GTK_POLICY_AUTOMATIC);
 
 	/* Manage auto save data */
 	auto_save = g_settings_get_boolean (tab->priv->editor,
@@ -1602,28 +1587,19 @@ gedit_tab_init (GeditTab *tab)
 	tab->priv->auto_save = (tab->priv->auto_save != FALSE);
 
 	tab->priv->auto_save_interval = auto_save_interval;
-	/* FIXME
-	if (tab->priv->auto_save_interval <= 0)
-		tab->priv->auto_save_interval = GPM_DEFAULT_AUTO_SAVE_INTERVAL;*/
 
-	/* Create the view */
-	doc = gedit_document_new ();
+	/* Create the frame */
+	tab->priv->frame = gedit_view_frame_new ();
+	gtk_widget_show (GTK_WIDGET (tab->priv->frame));
+
+	gtk_box_pack_end (GTK_BOX (tab), GTK_WIDGET (tab->priv->frame),
+	                  TRUE, TRUE, 0);
+
+	doc = gedit_view_frame_get_document (tab->priv->frame);
 	g_object_set_data (G_OBJECT (doc), GEDIT_TAB_KEY, tab);
 
-	_gedit_document_set_mount_operation_factory (doc,
-						     tab_mount_operation_factory,
-						     tab);
-
-	tab->priv->view = gedit_view_new (doc);
-	g_object_unref (doc);
-	gtk_widget_show (tab->priv->view);
-	g_object_set_data (G_OBJECT (tab->priv->view), GEDIT_TAB_KEY, tab);
-
-	gtk_box_pack_end (GTK_BOX (tab), sw, TRUE, TRUE, 0);
-	gtk_container_add (GTK_CONTAINER (sw), tab->priv->view);
-	gtk_scrolled_window_set_shadow_type (GTK_SCROLLED_WINDOW (sw),
-					     GTK_SHADOW_IN);	
-	gtk_widget_show (sw);
+	view = gedit_view_frame_get_view (tab->priv->frame);
+	g_object_set_data (G_OBJECT (view), GEDIT_TAB_KEY, tab);
 
 	g_signal_connect (doc,
 			  "notify::location",
@@ -1654,15 +1630,20 @@ gedit_tab_init (GeditTab *tab)
 			  G_CALLBACK (document_saved),
 			  tab);
 
-	g_signal_connect_after (tab->priv->view,
+	g_signal_connect_after (view,
 				"focus-in-event",
 				G_CALLBACK (view_focused_in),
 				tab);
 
-	g_signal_connect_after (tab->priv->view,
+	g_signal_connect_after (view,
 				"realize",
 				G_CALLBACK (view_realized),
 				tab);
+
+	g_signal_connect (view,
+			  "drop-uris",
+			  G_CALLBACK (on_drop_uris),
+			  tab);
 }
 
 GtkWidget *
@@ -1724,12 +1705,14 @@ _gedit_tab_new_from_stream (GInputStream        *stream,
  *
  * Gets the #GeditView inside @tab.
  *
- * Returns: the #GeditView inside @tab
+ * Returns: (transfer none): the #GeditView inside @tab
  */
 GeditView *
 gedit_tab_get_view (GeditTab *tab)
 {
-	return GEDIT_VIEW (tab->priv->view);
+	g_return_val_if_fail (GEDIT_IS_TAB (tab), NULL);
+
+	return gedit_view_frame_get_view (tab->priv->frame);
 }
 
 /**
@@ -1738,13 +1721,14 @@ gedit_tab_get_view (GeditTab *tab)
  *
  * Gets the #GeditDocument associated to @tab.
  *
- * Returns: the #GeditDocument associated to @tab
+ * Returns: (transfer none): the #GeditDocument associated to @tab
  */
 GeditDocument *
 gedit_tab_get_document (GeditTab *tab)
 {
-	return GEDIT_DOCUMENT (gtk_text_view_get_buffer (
-					GTK_TEXT_VIEW (tab->priv->view)));
+	g_return_val_if_fail (GEDIT_IS_TAB (tab), NULL);
+
+	return gedit_view_frame_get_document (tab->priv->frame);
 }
 
 #define MAX_DOC_NAME_LENGTH 40
@@ -1793,7 +1777,7 @@ _gedit_tab_get_name (GeditTab *tab)
 }
 
 gchar *
-_gedit_tab_get_tooltips	(GeditTab *tab)
+_gedit_tab_get_tooltip (GeditTab *tab)
 {
 	GeditDocument *doc;
 	gchar *tip;
@@ -2066,7 +2050,7 @@ _gedit_tab_get_icon (GeditTab *tab)
  *
  * Gets the #GeditTab associated with @doc.
  *
- * Returns: the #GeditTab associated with @doc
+ * Returns: (transfer none): the #GeditTab associated with @doc
  */
 GeditTab *
 gedit_tab_get_from_document (GeditDocument *doc)
@@ -2161,7 +2145,7 @@ _gedit_tab_revert (GeditTab *tab)
 
 	if (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION)
 	{
-		set_message_area (tab, NULL);
+		set_info_bar (tab, NULL);
 	}
 
 	doc = gedit_tab_get_document (tab);
@@ -2208,11 +2192,11 @@ _gedit_tab_save (GeditTab *tab)
 	if (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION)
 	{
 		/* We already told the user about the external
-		 * modification: hide the message area and set
+		 * modification: hide the message bar and set
 		 * the save flag.
 		 */
 
-		set_message_area (tab, NULL);
+		set_info_bar (tab, NULL);
 		save_flags = tab->priv->save_flags | GEDIT_DOCUMENT_SAVE_IGNORE_MTIME;
 	}
 	else
@@ -2327,11 +2311,11 @@ _gedit_tab_save_as (GeditTab                     *tab,
 	if (tab->priv->state == GEDIT_TAB_STATE_EXTERNALLY_MODIFIED_NOTIFICATION)
 	{
 		/* We already told the user about the external
-		 * modification: hide the message area and set
+		 * modification: hide the message bar and set
 		 * the save flag.
 		 */
 
-		set_message_area (tab, NULL);
+		set_info_bar (tab, NULL);
 		save_flags = tab->priv->save_flags | GEDIT_DOCUMENT_SAVE_IGNORE_MTIME;
 	}
 	else
@@ -2402,20 +2386,20 @@ get_print_settings (GeditTab *tab)
 	}
 }
 
-/* FIXME: show the message area only if the operation will be "long" */
+/* FIXME: show the info bar only if the operation will be "long" */
 static void
 printing_cb (GeditPrintJob       *job,
 	     GeditPrintJobStatus  status,
 	     GeditTab            *tab)
 {
-	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));	
+	g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->priv->info_bar));	
 
-	gtk_widget_show (tab->priv->message_area);
+	gtk_widget_show (tab->priv->info_bar);
 
-	gedit_progress_message_area_set_text (GEDIT_PROGRESS_MESSAGE_AREA (tab->priv->message_area),
+	gedit_progress_info_bar_set_text (GEDIT_PROGRESS_INFO_BAR (tab->priv->info_bar),
 					      gedit_print_job_get_status_string (job));
 
-	gedit_progress_message_area_set_fraction (GEDIT_PROGRESS_MESSAGE_AREA (tab->priv->message_area),
+	gedit_progress_info_bar_set_fraction (GEDIT_PROGRESS_INFO_BAR (tab->priv->info_bar),
 						  gedit_print_job_get_progress (job));
 }
 
@@ -2478,9 +2462,9 @@ done_printing_cb (GeditPrintJob       *job,
 	}
 	else
 	{
-		g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
+		g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->priv->info_bar));
 
-		set_message_area (tab, NULL); /* destroy the message area */
+		set_info_bar (tab, NULL); /* destroy the info bar */
 	}
 
 	/* TODO: check status and error */
@@ -2545,7 +2529,7 @@ show_preview_cb (GeditPrintJob       *job,
 	/* g_return_if_fail (tab->priv->state == GEDIT_TAB_STATE_PRINT_PREVIEWING); */
 	g_return_if_fail (tab->priv->print_preview == NULL);
 
-	set_message_area (tab, NULL); /* destroy the message area */
+	set_info_bar (tab, NULL); /* destroy the info bar */
 
 	tab->priv->print_preview = GTK_WIDGET (preview);
 	gtk_box_pack_end (GTK_BOX (tab),
@@ -2599,8 +2583,8 @@ preview_finished_cb (GtkSourcePrintJob *pjob, GeditTab *tab)
 	GnomePrintJob *gjob;
 	GtkWidget *preview = NULL;
 
-	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
-	set_message_area (tab, NULL); /* destroy the message area */
+	g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->priv->info_bar));
+	set_info_bar (tab, NULL); /* destroy the info bar */
 	
 	gjob = gtk_source_print_job_get_print_job (pjob);
 
@@ -2619,11 +2603,11 @@ preview_finished_cb (GtkSourcePrintJob *pjob, GeditTab *tab)
 #endif
 
 static void
-print_cancelled (GtkWidget        *area,
+print_cancelled (GtkWidget        *bar,
                  gint              response_id,
                  GeditTab         *tab)
 {
-	g_return_if_fail (GEDIT_IS_PROGRESS_MESSAGE_AREA (tab->priv->message_area));
+	g_return_if_fail (GEDIT_IS_PROGRESS_INFO_BAR (tab->priv->info_bar));
 
 	gedit_print_job_cancel (tab->priv->print_job);
 
@@ -2631,77 +2615,31 @@ print_cancelled (GtkWidget        *area,
 }
 
 static void
-show_printing_message_area (GeditTab *tab, gboolean preview)
+show_printing_info_bar (GeditTab *tab, gboolean preview)
 {
-	GtkWidget *area;
+	GtkWidget *bar;
 
 	if (preview)
 	{
-		area = gedit_progress_message_area_new (GTK_STOCK_PRINT_PREVIEW,
+		bar = gedit_progress_info_bar_new (GTK_STOCK_PRINT_PREVIEW,
 							"",
 							TRUE);
 	}
 	else
 	{
-		area = gedit_progress_message_area_new (GTK_STOCK_PRINT,
+		bar = gedit_progress_info_bar_new (GTK_STOCK_PRINT,
 							"",
 							TRUE);
 	}
 
-	g_signal_connect (area,
+	g_signal_connect (bar,
 			  "response",
 			  G_CALLBACK (print_cancelled),
 			  tab);
 	  
-	set_message_area (tab, area);
+	set_info_bar (tab, bar);
 }
 
-#if !GTK_CHECK_VERSION (2, 17, 4)
-
-static void
-page_setup_done_cb (GtkPageSetup *setup,
-		    GeditTab     *tab)
-{
-	if (setup != NULL)
-	{
-		GeditDocument *doc;
-
-		doc = gedit_tab_get_document (tab);
-
-		/* remember it for this document */
-		g_object_set_data_full (G_OBJECT (doc),
-					GEDIT_PAGE_SETUP_KEY,
-					g_object_ref (setup),
-					(GDestroyNotify)g_object_unref);
-
-		/* make it the default */
-		_gedit_app_set_default_page_setup (gedit_app_get_default(),
-						   setup);
-	}
-}
-
-void 
-_gedit_tab_page_setup (GeditTab *tab)
-{
-	GtkPageSetup *setup;
-	GtkPrintSettings *settings;
-
-	g_return_if_fail (GEDIT_IS_TAB (tab));
-
-	setup = get_page_setup (tab);
-	settings = get_print_settings (tab);
-
-	gtk_print_run_page_setup_dialog_async (GTK_WINDOW (gtk_widget_get_toplevel (GTK_WIDGET (tab))),
-		 			       setup,
-		 			       settings,
-					       (GtkPageSetupDoneFunc) page_setup_done_cb,
-					       tab);
-
-	/* CHECK: should we unref setup and settings? */
-}
-
-#endif
-
 static void
 gedit_tab_print_or_print_preview (GeditTab                *tab,
 				  GtkPrintOperationAction  print_action)
@@ -2724,7 +2662,7 @@ gedit_tab_print_or_print_preview (GeditTab                *tab,
 	g_object_add_weak_pointer (G_OBJECT (tab->priv->print_job), 
 				   (gpointer *) &tab->priv->print_job);
 
-	show_printing_message_area (tab, is_preview);
+	show_printing_info_bar (tab, is_preview);
 
 	g_signal_connect (tab->priv->print_job,
 			  "printing",
@@ -2860,7 +2798,7 @@ gedit_tab_get_auto_save_enabled	(GeditTab *tab)
  **/
 void
 gedit_tab_set_auto_save_enabled	(GeditTab *tab, 
-				 gboolean enable)
+				 gboolean  enable)
 {
 	GeditDocument *doc = NULL;
 	GeditLockdownMask lockdown;
@@ -2942,7 +2880,7 @@ gedit_tab_get_auto_save_interval (GeditTab *tab)
  **/
 void 
 gedit_tab_set_auto_save_interval (GeditTab *tab, 
-				  gint interval)
+				  gint      interval)
 {
 	GeditDocument *doc = NULL;
 	
@@ -2982,7 +2920,13 @@ gedit_tab_set_info_bar (GeditTab  *tab,
 	g_return_if_fail (info_bar == NULL || GTK_IS_WIDGET (info_bar));
 
 	/* FIXME: this can cause problems with the tab state machine */
-	set_message_area (tab, info_bar);
+	set_info_bar (tab, info_bar);
+}
+
+GtkWidget *
+_gedit_tab_get_view_frame (GeditTab *tab)
+{
+	return GTK_WIDGET (tab->priv->frame);
 }
 
 /* ex:ts=8:noet: */
