@@ -368,6 +368,8 @@ gedit_file_browser_widget_finalize (GObject *object)
 
 	gdk_cursor_unref (obj->priv->busy_cursor);
 
+	g_free (obj->priv->filter_pattern_str);
+
 	G_OBJECT_CLASS (gedit_file_browser_widget_parent_class)->finalize (object);
 }
 
@@ -430,7 +432,7 @@ gedit_file_browser_widget_class_init (GeditFileBrowserWidgetClass *klass)
 					 g_param_spec_string ("filter-pattern",
 					 		      "Filter Pattern",
 					 		      "The filter pattern",
-					 		      NULL,
+					 		      "",
 					 		      G_PARAM_READWRITE));
 	g_object_class_install_property (object_class, PROP_ENABLE_DELETE,
 					 g_param_spec_boolean ("enable-delete",
@@ -686,7 +688,8 @@ insert_location_path (GeditFileBrowserWidget *obj)
 							   obj);
 		}
 
-		if (g_file_equal (current, loc->root) || !gedit_utils_file_has_parent (current))
+		if (g_file_equal (current, loc->root) ||
+		    !g_file_has_parent (current, NULL))
 		{
 			if (current != loc->virtual_root)
 				g_object_unref (current);
@@ -1281,6 +1284,7 @@ gedit_file_browser_widget_init (GeditFileBrowserWidget *obj)
 {
 	obj->priv = GEDIT_FILE_BROWSER_WIDGET_GET_PRIVATE (obj);
 
+	obj->priv->filter_pattern_str = g_strdup ("");
 	obj->priv->bookmarks_hash = g_hash_table_new_full (g_file_hash,
 			                                   (GEqualFunc)g_file_equal,
 			                                   g_object_unref,
@@ -1780,18 +1784,25 @@ set_filter_pattern_real (GeditFileBrowserWidget *obj,
 	if (pattern != NULL && *pattern == '\0')
 		pattern = NULL;
 
-	if (pattern == NULL && obj->priv->filter_pattern_str == NULL)
+	if (pattern == NULL && *obj->priv->filter_pattern_str == '\0')
 		return;
 
-	if (pattern != NULL && obj->priv->filter_pattern_str != NULL &&
-	    strcmp (pattern, obj->priv->filter_pattern_str) == 0)
+	if (pattern != NULL && strcmp (pattern, obj->priv->filter_pattern_str) == 0)
 	{
 		return;
 	}
 
 	/* Free the old pattern */
 	g_free (obj->priv->filter_pattern_str);
-	obj->priv->filter_pattern_str = g_strdup (pattern);
+
+	if (pattern == NULL)
+	{
+		obj->priv->filter_pattern_str = g_strdup ("");
+	}
+	else
+	{
+		obj->priv->filter_pattern_str = g_strdup (pattern);
+	}
 
 	if (obj->priv->filter_pattern)
 	{
@@ -1824,18 +1835,13 @@ set_filter_pattern_real (GeditFileBrowserWidget *obj,
 
 	if (update_entry)
 	{
-		if (obj->priv->filter_pattern_str == NULL)
-		{
-			gtk_entry_set_text (GTK_ENTRY (obj->priv->filter_entry),
-			                    "");
-		}
-		else
-		{
-			gtk_entry_set_text (GTK_ENTRY (obj->priv->filter_entry),
-			                    obj->priv->filter_pattern_str);
+		gtk_entry_set_text (GTK_ENTRY (obj->priv->filter_entry),
+		                    obj->priv->filter_pattern_str);
 
+		if (obj->priv->filter_pattern_str != '\0')
+		{
 			gtk_expander_set_expanded (GTK_EXPANDER (obj->priv->filter_expander),
-		        	                   TRUE);
+			                           TRUE);
 		}
 	}
 
@@ -2809,7 +2815,7 @@ do_change_directory (GeditFileBrowserWidget *obj,
 
 	if ((event->state &
 	    (~GDK_CONTROL_MASK & ~GDK_SHIFT_MASK & ~GDK_MOD1_MASK)) ==
-	     event->state && event->keyval == GDK_BackSpace)
+	     event->state && event->keyval == GDK_KEY_BackSpace)
 	{
 		action = gtk_action_group_get_action (obj->priv->action_group_sensitive,
 		                                      "DirectoryPrevious");
@@ -2822,15 +2828,15 @@ do_change_directory (GeditFileBrowserWidget *obj,
 
 	switch (event->keyval)
 	{
-		case GDK_Left:
+		case GDK_KEY_Left:
 			action = gtk_action_group_get_action (obj->priv->action_group_sensitive,
 			                                      "DirectoryPrevious");
 			break;
-		case GDK_Right:
+		case GDK_KEY_Right:
 			action = gtk_action_group_get_action (obj->priv->action_group_sensitive,
 			                                      "DirectoryNext");
 			break;
-		case GDK_Up:
+		case GDK_KEY_Up:
 			action = gtk_action_group_get_action (obj->priv->action_group,
 			                                      "DirectoryUp");
 			break;
@@ -2865,7 +2871,8 @@ on_treeview_key_press_event (GeditFileBrowserView   *treeview,
 
 	modifiers = gtk_accelerator_get_default_mod_mask ();
 
-	if (event->keyval == GDK_Delete || event->keyval == GDK_KP_Delete)
+	if (event->keyval == GDK_KEY_Delete ||
+	    event->keyval == GDK_KEY_KP_Delete)
 	{
 
 		if ((event->state & modifiers) == GDK_SHIFT_MASK &&
@@ -2881,7 +2888,7 @@ on_treeview_key_press_event (GeditFileBrowserView   *treeview,
 		}
 	}
 
-	if ((event->keyval == GDK_F2) && (event->state & modifiers) == 0)
+	if ((event->keyval == GDK_KEY_F2) && (event->state & modifiers) == 0)
 	{
 		rename_selected_file (obj);
 		return TRUE;
